@@ -51,7 +51,7 @@ SCHEMA_KEYS = tuple(
 # Note that in some cases ("dependencies"), the value may be a list of strings.
 SCHEMA_OBJECT_KEYS = ("properties", "patternProperties", "dependencies")
 ALL_KEYWORDS = tuple(
-    ["$schema", *SCHEMA_KEYS, *SCHEMA_OBJECT_KEYS]
+    [*SCHEMA_KEYS, *SCHEMA_OBJECT_KEYS]
     + sum((s.split() for _, s in TYPE_SPECIFIC_KEYS), [])
 )
 
@@ -394,13 +394,12 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
         schema.pop("items", None)
     if (
         "properties" in schema
-        and "maxProperties" in schema
         and not schema.get("patternProperties")
         and schema.get("additionalProperties") == FALSEY
     ):
-        schema["maxProperties"] = min(
-            schema["maxProperties"], len(schema["properties"])
-        )
+        max_props = schema.get("maxProperties", math.inf)
+        assert isinstance(max_props, (int, float))
+        schema["maxProperties"] = min(max_props, len(schema["properties"]))
     if "object" in type_ and schema.get("minProperties", 0) > schema.get(
         "maxProperties", math.inf
     ):
@@ -551,7 +550,9 @@ class LocalResolver(jsonschema.RefResolver):
         )
 
 
-def resolve_all_refs(schema: Schema, *, resolver: LocalResolver = None) -> Schema:
+def resolve_all_refs(
+    schema: Union[bool, Schema], *, resolver: LocalResolver = None
+) -> Schema:
     """
     Resolve all references in the given schema.
 
@@ -559,6 +560,8 @@ def resolve_all_refs(schema: Schema, *, resolver: LocalResolver = None) -> Schem
     The latter require special handling to convert to strategies and are much
     less common, so we just ignore them (and error out) for now.
     """
+    if isinstance(schema, bool):
+        return canonicalish(schema)
     if resolver is None:
         resolver = LocalResolver.from_schema(deepcopy(schema))
     if not isinstance(resolver, jsonschema.RefResolver):
@@ -573,7 +576,7 @@ def resolve_all_refs(schema: Schema, *, resolver: LocalResolver = None) -> Schem
             if s == {}:
                 return resolve_all_refs(got, resolver=resolver)
             m = merged([s, got])
-            if m is None:
+            if m is None:  # pragma: no cover
                 msg = f"$ref:{ref!r} had incompatible base schema {s!r}"
                 raise HypothesisRefResolutionError(msg)
             return resolve_all_refs(m, resolver=resolver)
@@ -737,15 +740,26 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
                     out["multipleOf"] = max(x, y)
                 else:
                     return None
+        # TODO: merge `contains` schemas when one is a subset of the other
+        # TODO: merge `items` schemas or lists-of-schemas
+        # TODO: merge `not` schemas as {not: anyOf: [not1, not2]}
+        # TODO: merge if/then/else schemas to the chained form
+        #       or maybe canonicalise them to an anyOf instead?
+        # TODO: merge dependencies
 
+        # This loop handles the remaining cases.  Notably, we do not attempt to
+        # merge distinct values for:
+        # - `pattern`; computing regex intersection is out of scope
+        # - `contains`; requires allOf and thus enters an infinite loop
+        # - `$ref`; if not already resolved we can't do that here
+        # - `anyOf`; due to product-like explosion in worst case
+        # - `oneOf`; which we plan to handle as an anyOf-not composition
         for k, v in s.items():
             if k not in out:
                 out[k] = v
             elif out[k] != v and k in ALL_KEYWORDS:
                 # If non-validation keys like `title` or `description` don't match,
                 # that doesn't really matter and we'll just go with first we saw.
-                known = "$ref $schema anyOf oneOf items if not pattern contains"
-                assert k in known.split(), (k, out[k], v)
                 return None
         out = canonicalish(out)
         if out == FALSEY:
